require "cutest"
require "cuba"
require "cuba/test"
require_relative "../lib/cuba/csrf"

def assert_no_raise
  yield
  success
end

scope do
  setup do
    Cuba.reset!

    Cuba.use(Rack::Session::Cookie, secret: "_this_must_be_secret")
    Cuba.plugin(Cuba::CSRF)
  end

  test "safe http methods" do
    Cuba.define do
      raise unless csrf_safe?
    end

    assert_no_raise do
      get  "/"
      head "/"
    end
  end

  test "invalid csrf param" do
    Cuba.define do
      raise unless csrf_safe?
    end

    assert_raise do
      post "/", "csrf_token" => nil
    end

    assert_raise do
      post "/", "csrf_token" => ""
    end

    assert_raise do
      post "/", "csrf_token" => "nonsense"
    end
  end

  test "valid csrf param" do
    Cuba.define do
      raise unless csrf_safe?

      on get do
        res.write(csrf_token)
      end

      on post do
        res.write("safe")
      end
    end

    get "/"

    csrf_token = last_response.body

    assert(!csrf_token.empty?)

    assert_no_raise do
      post "/", "csrf_token" => csrf_token
    end
  end

  test "http header" do
    csrf_token = SecureRandom.hex(32)

    Cuba.define do
      session[:csrf_token] = csrf_token
      raise unless csrf_safe?
    end

    post "/", {}, { "HTTP_X_CSRF_TOKEN" => csrf_token }
  end

  test "sub app raises too" do
    class App < Cuba
      define do
        on post do
          res.write("unsafe")
        end
      end
    end

    Cuba.define do
      raise unless csrf_safe?

      on "app" do
        run(App)
      end
    end

    assert_raise do
      post "/app"
    end
  end

  test "only sub app" do
    class App < Cuba
      define do
        protect_from_forgery!

        on post do
          res.write("unsafe")
        end
      end
    end

    Cuba.define do
      on "app" do
        run(App)
      end

      on default do
        res.write("safe")
      end
    end

    assert_no_raise do
      post "/"
    end

    assert_raise do
      post "/app"
    end
  end

  test "html helpers" do
    Cuba.plugin(Cuba::CSRF)

    class Api < Cuba
      def session
        @session ||= {}
      end
    end

    api = Api.new

    csrf_form_tag = %Q(<input type="hidden" name="csrf_token" value="#{api.csrf_token}">)
    csrf_meta_tag = %Q(<meta name="csrf_token" content="#{api.csrf_token}">)

    assert_equal(csrf_form_tag, api.csrf_tag)
    assert_equal(csrf_form_tag, api.csrf_form_tag)
    assert_equal(csrf_meta_tag, api.csrf_meta_tag)
  end
end
